@@ -31,7 +31,7 @@ void YouTube::login(const QString &username, const QString &password) {
     setCurrentUser(username);
     QNetworkRequest request(QUrl("https://www.google.com/accounts/ClientLogin"));
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-    QByteArray loginData("Email=" + username.toAscii() + "&Passwd=" + password.toAscii() + "&service=youtube&source=cuteTube");
+    QByteArray loginData("Email=" + username.toAscii().toPercentEncoding() + "&Passwd=" + password.toAscii().toPercentEncoding() + "&service=youtube&source=cuteTube");
     QNetworkReply* reply = nam->post(request, loginData);
     connect(reply, SIGNAL(finished()), this, SLOT(checkLogin()));
 }
@@ -80,10 +80,10 @@ void YouTube::uploadVideo(const QString &filename, const QString &title, const Q
                    "xmlns:media=\"http://search.yahoo.com/mrss/\"\n" \
                    "xmlns:yt=\"http://gdata.youtube.com/schemas/2007\">\n" \
                    "<media:group>\n" \
-                   "<media:title>" + title.toAscii() + "</media:title>\n" \
-                   "<media:description>\n" + description.toAscii() + "\n\n" + "Uploaded via cuteTube on the Nokia N900\n</media:description>\n" \
+                   "<media:title>" + title.toAscii().toPercentEncoding() + "</media:title>\n" \
+                   "<media:description>\n" + description.toAscii().toPercentEncoding() + "\n\n" + "Uploaded via cuteTube on the Nokia N900\n</media:description>\n" \
                    "<media:category scheme=\"http://gdata.youtube.com/schemas/2007/categories.cat\">\n" + category.toAscii() + "\n</media:category>\n" \
-                   "<media:keywords>" + tags.toAscii() + "</media:keywords>\n" \
+                   "<media:keywords>" + tags.toAscii().toPercentEncoding() + "</media:keywords>\n" \
                    "</media:group>\n" \
                    "</entry>");
 
@@ -218,13 +218,18 @@ void YouTube::postFinished() {
 
     int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
     QByteArray statusText = reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toByteArray();
-    //qDebug() << "Status is:" << statusCode << ":" << statusText;
+//    qDebug() << "Status is:" << statusCode << ":" << statusText;
     if ((statusCode == 200) || (statusCode == 201)) {
         emit postSuccessful();
     }
+    else if ((statusText == "Bad Request") || ((statusText == "Forbidden") && !(currentUser == ""))) {
+        emit postFailed();
+    }
     else {
         emit alert(tr("Error - Server repsonse is: ") + statusText);
     }
+    disconnect(this, SIGNAL(postSuccessful()), 0, 0);
+    disconnect(this, SIGNAL(postFailed()), 0, 0);
     reply->deleteLater();
 }
 
@@ -249,6 +254,7 @@ void YouTube::addToFavourites(const QString &videoId) {
                    "</entry>");
     postRequest(url, xml);
     connect(this, SIGNAL(postSuccessful()), this, SIGNAL(addedToFavourites()));
+    connect(this, SIGNAL(postFailed()), this, SIGNAL(videoInFavourites()));
 }
 
 void YouTube::deleteFromFavourites(const QString &favouriteId) {
@@ -279,8 +285,8 @@ void YouTube::createNewPlaylist(const QString &title, const QString &description
     QByteArray xml("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" \
                    "<entry xmlns=\"http://www.w3.org/2005/Atom\"\n" \
                    "xmlns:yt=\"http://gdata.youtube.com/schemas/2007\">\n" \
-                   "<title>" + title.toAscii() + "</title>\n" \
-                   "<summary>" + description.toAscii() + "</summary>\n" \
+                   "<title>" + title.toAscii().toPercentEncoding() + "</title>\n" \
+                   "<summary>" + description.toAscii().toPercentEncoding() + "</summary>\n" \
                    "</entry>");
     if (isPrivate) {
         int index = xml.lastIndexOf("<");
@@ -293,6 +299,7 @@ void YouTube::createNewPlaylist(const QString &title, const QString &description
 void YouTube::deletePlaylist(const QString &playlistId) {
     QUrl url("http://gdata.youtube.com/feeds/api/users/" + currentUser + "/playlists/" + playlistId);
     deleteRequest(url);
+    connect(this, SIGNAL(postSuccessful()), this, SIGNAL(playlistDeleted()));
 }
 
 void YouTube::subscribeToChannel(const QString &username) {
@@ -323,6 +330,7 @@ void YouTube::rateVideo(const QString &videoId, const QString &likeOrDislike) {
                    "</entry>");
     postRequest(url, xml);
     connect(this, SIGNAL(postSuccessful()), this, SIGNAL(videoRated()));
+    connect(this, SIGNAL(postFailed()), this, SIGNAL(cannotRate()));
 }
 
 void YouTube::addComment(const QString &videoId, const QString &comment) {
@@ -330,7 +338,7 @@ void YouTube::addComment(const QString &videoId, const QString &comment) {
     QByteArray xml("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" \
                    "<entry xmlns=\"http://www.w3.org/2005/Atom\"\n" \
                    "xmlns:yt=\"http://gdata.youtube.com/schemas/2007\">\n" \
-                   "<content>" + comment.toAscii() + "</content>\n" \
+                   "<content>" + comment.toAscii().toPercentEncoding() + "</content>\n" \
                    "</entry>");
     postRequest(url, xml);
     connect(this, SIGNAL(postSuccessful()), this, SIGNAL(commentAdded()));
@@ -344,7 +352,7 @@ void YouTube::replyToComment(const QString &videoId, const QString &commentId, c
                    "<link rel=\"http://gdata.youtube.com/schemas/2007#in-reply-to\"\n" \
                    "type=\"application/atom+xml\"\n" \
                    "href=\"http://gdata.youtube.com/feeds/api/videos/" + videoId.toAscii() + "/comments/" + commentId.toAscii() + "\"/>\n" \
-                   "<content>" + comment.toAscii() + "</content>\n" \
+                   "<content>" + comment.toAscii().toPercentEncoding() + "</content>\n" \
                    "</entry>");
     postRequest(url, xml);
 }
@@ -376,7 +384,7 @@ void YouTube::parseVideoPage(QNetworkReply *reply) {
     for (int i = 1; i < parts.length(); i++) {
         QByteArray part = parts[i];
         QList<QByteArray> keyAndValue = part.split(',');
-        QByteArray url = keyAndValue.first();
+        QByteArray url = keyAndValue.first().replace("%2C", ",");
         formats[key] = url;
         key = keyAndValue.last().toInt();
     }
@@ -400,44 +408,4 @@ void YouTube::parseVideoPage(QNetworkReply *reply) {
     manager->deleteLater();
 }
 
-void YouTube::parseVideoPage() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
-    if (!reply) {
-        return;
-    }
-
-    QMap<int, QByteArray> formats;
-    QByteArray response = reply->readAll();
-//    qDebug() << response;
-    response = response.simplified().replace(QByteArray(" "), QByteArray(""));
-    int pos = response.indexOf("\",\"fmt_url_map\":\"") + 17;
-    int pos2 = response.indexOf('\"', pos);
-    response = response.mid(pos, pos2 - pos);
-    QList<QByteArray> parts = response.split('|');
-    int key = parts.first().toInt();
-    for (int i = 1; i < parts.length(); i++) {
-        QByteArray part = parts[i];
-        QList<QByteArray> keyAndValue = part.split(',');
-        QByteArray url = keyAndValue.first().replace(QByteArray("\\/"), QByteArray("/")).replace(QByteArray("\\u0026"), QByteArray("&"));
-        formats[key] = url;
-        key = keyAndValue.last().toInt();
-    }
-    QList<int> flist;
-    flist << 22 << 35 << 34 << 18 << 5;
-    QByteArray videoUrl = "";
-    int index = flist.indexOf(playbackFormat);
-    while ((videoUrl == "") && index < flist.size()) {
-        videoUrl = formats.value(flist.at(index), "");
-        index++;
-    }
-    if (videoUrl == "") {
-        emit alert(tr("Error: Unable to retrieve video"));
-        emit videoUrlError();
-    }
-    else {
-        emit gotVideoUrl(QString(videoUrl));
-    }
-//    qDebug() << videoUrl;
-    reply->deleteLater();
-}
 
